var redis = require('redis');
var qbox = require('qbox');
var EventEmitter = require('events').EventEmitter;

module.exports = new Drev();

function Drev() {

	var self = this;
	var pub, sub;
	var connection = qbox.create(['pub', 'sub']);
	var events = new EventEmitter();
	var connected = false;

	var subscriptions = {};

	this.on = function(event, callback) {

		if(isSelfEvent(event)) {
			events.on(event, callback);
		} else {

			connection.ready(function() {
				
				events.on(event, callback);
				//subscribe only if not subscribed earlier
				if(!subscriptions[event]) {
					sub.subscribe(event);
					subscriptions[event] = true;
				}
			});
		}
	};

	this.emit = function() {

		var arguments_ = arguments;
		
		connection.ready(function() {
			var channel = arguments_[0];
			var args = [];

			for(var index in arguments_) {
				if(index!=0) {
					args.push(arguments_[index]);					
				}	
			}

			pub.publish(channel, JSON.stringify(args));
		});
	};

	this.once = function(event, callback) {
		
		connection.ready(function() {
			
			events.once(event, callback);
			//subscribe only if not subscribed earlier
			if(!subscriptions[event]) {
				sub.subscribe(event);
				subscriptions[event] = true;
			}

		});
	};

	this.removeListener = function(event, listener) {
		events.removeListener(event, listener);	
	};

	this.removeAllListeners = function(event) {
		events.removeAllListeners(event);
		sub.unsubscribe(event);
		delete subscriptions[event];
	};

	this.listeners = function(event) {
		return events.listeners(event);
	};

	this.setMaxListeners = function(n) {
		return events.setMaxListeners(n);
	};

	/**
		Start the Connection to the Client
	*/
	this.start = function(host, port, options) {

		//if connected there is no reason to connect again
		if(connected) return;

		//reset subscriptions
		connection.reset();
		subscriptions = {};
		var reconnection = null;

		pub = redis.createClient(host, port, options);
		sub = redis.createClient(host, port, options);
		
		pub.on('connect', function() { 
			connection.tick('pub');
		});
		sub.on('connect', function() { 
			connection.tick('sub');
		});

		pub.on('error', function(err) {
			
			connected = false;
			connection.reset(); //reset the qbox ticking for the reconnect
			events.emit('self::error', err); 
		});
		sub.on('error', function(err) { 

			connected = false;
			events.emit('self::error', err); 
		});

		//event triggred when connected at initially and after every reconnect (An Qox Meth)
		connection.onStart(function() { 
			
			connected = true;			
			events.emit('self::connected'); 
			//reconnect all the subscriptions back
			for(var event in subscriptions) {
				sub.subscribe(event);
			}
		});

		sub.on('message', function(channel, message) {
			
			var args;
			try{
				args = JSON.parse(message);
				args = (args instanceof Array)? args: [args];
			} catch(err) {
				args = ['error data format']; 
			}

			events.emit.apply(events, [channel].concat(args));

		});
	}

	/**
		Stop the connection to the redis
		@returns true if disconneted or false if already disconnected
	*/
	this.stop = function() {
		
		if(connected) {
			connected = false;	
			event = new EventEmitter();
			pub.quit();
			sub.quit();
			return true;	
		}

		return false;
	};

	function isSelfEvent(event) {
		return event.substr(0,4) == 'self';
	}
}